@@ -198,8 +198,8 @@ def test_encode_symbols():
     assert s == decoded
 
 
-def test_encode_long_neg_conversion():
-    test_input = -9223372036854775808
+@pytest.mark.parametrize("test_input", [-1, -9223372036854775808, 18446744073709551615])
+def test_encode_special_longs(test_input):
     output = ujson.encode(test_input)
 
     json.loads(output)
@@ -242,7 +242,7 @@ def test_encode_dict_values_ref_counting():
 @pytest.mark.skipif(
     hasattr(sys, "pypy_version_info"), reason="PyPy uses incompatible GC"
 )
-@pytest.mark.parametrize("key", ["key", b"key", 1, True, None])
+@pytest.mark.parametrize("key", ["key", b"key", 1, True, False, None])
 @pytest.mark.parametrize("sort_keys", [False, True])
 def test_encode_dict_key_ref_counting(key, sort_keys):
     import gc
@@ -495,6 +495,30 @@ def test_object_with_json_attribute_error():
         ujson.encode(d)
 
 
+def test_object_with_to_dict_type_error():
+    # toDict must return a dict, otherwise it should raise an error.
+    for return_value in (None, 1234, 12.34, True, "json"):
+
+        class JSONTest:
+            def toDict(self):
+                return return_value
+
+        d = {"key": JSONTest()}
+        with pytest.raises(TypeError):
+            ujson.encode(d)
+
+
+def test_object_with_to_dict_attribute_error():
+    # If toDict raises an error, make sure python actually raises it.
+    class JSONTest:
+        def toDict(self):
+            raise AttributeError
+
+    d = {"key": JSONTest()}
+    with pytest.raises(AttributeError):
+        ujson.encode(d)
+
+
 def test_decode_array_empty():
     test_input = "[]"
     obj = ujson.decode(test_input)
@@ -552,6 +576,13 @@ def test_sort_keys():
     assert sorted_keys == '{"a":1,"b":1,"c":1,"d":1,"e":1,"f":1}'
 
 
+def test_sort_keys_unordered():
+    data = {"a": 1, 1: 2, None: 3}
+    assert ujson.dumps(data) == '{"a":1,"1":2,"null":3}'
+    with pytest.raises(TypeError):
+        ujson.dumps(data, sort_keys=True)
+
+
 @pytest.mark.parametrize(
     "test_input",
     [
@@ -975,9 +1006,13 @@ def test_reject_bytes_false():
     assert ujson.dumps(data, reject_bytes=False) == '{"a":"b"}'
 
 
-def test_encode_none_key():
-    data = {None: None}
-    assert ujson.dumps(data) == '{"null":null}'
+def test_encode_special_keys():
+    data = {None: 0, True: 1, False: 2}
+    assert ujson.dumps(data) == '{"null":0,"true":1,"false":2}'
+    data = {None: 0}
+    assert ujson.dumps(data, sort_keys=True) == '{"null":0}'
+    data = {True: 1, False: 2}
+    assert ujson.dumps(data, sort_keys=True) == '{"false":2,"true":1}'
 
 
 def test_default_function():
@@ -1069,8 +1104,12 @@ def test_obj_str_exception(sort_keys):
         def __str__(self):
             raise NotImplementedError
 
+    key = Obj()
+    getrefcount = getattr(sys, "getrefcount", lambda x: 0)
+    old = getrefcount(key)
     with pytest.raises(NotImplementedError):
-        ujson.dumps({Obj(): 1}, sort_keys=sort_keys)
+        ujson.dumps({key: 1}, sort_keys=sort_keys)
+    assert getrefcount(key) == old
 
 
 def no_memory_leak(func_code, n=None):
